__author__ = "Felix Kleinert"
__date__ = '2020-04-07'

from src.run_modules.run_environment import RunEnvironment


class PartitionCheck(RunEnvironment):
    """
    Checking if running on a HPC login node. The only reason to run on login nodes is to download data.
    Training and validation should happen on compute nodes

    Note: This Method is highly customised to the HCP-systems in Juelich (FZJ, JSC). When using an other HPC system,
    make sure to double check the indexing of `self.data_store.get('hostname')'.
    """

    def __init__(self):
        # create run framework
        super().__init__()

        self._run()

    def _run(self):
        if (self.data_store.get('hostname')[:2] in self.data_store.get('login_nodes')) or (
                self.data_store.get('hostname')[:6] in self.data_store.get('login_nodes')):
            raise OSError(
                'You are on a login node to download data. Use compute nodes and run again if you want to train and validate a model.')
